// Author:       Andreas F. Borchert
// Description:  template class for histories,
//               i.e. lists that store only the last n items added to it

#ifndef HISTORY_H
#define HISTORY_H

#include <vector>

template<typename Item>
class History {
   public:
      // constructor
      History(int nitems);

      // accessors
      int max_size() const; // returns capacity
      int size() const; // returns # of items in buffer
      const Item& operator[](int i) const;
	 // PRE: i >= 0 && i < size()
	 // i = 0: return item added last
	 // i = 1: return item before last item

      // mutators
      void add(const Item& item);

   private:
      std::vector< Item > items; // ring buffer with the last n items
      int index; // next item will be stored at items[index]
      int nof_items; // # of items in ring buffer so far
};

#endif
