// Author:       Andreas F. Borchert
// Description:  template class for histories,
//               i.e. lists that store only the last n items added to it

#include <cassert>
#include "History.h"

template<typename Item>
History<Item>::History(int nitems) :
      items(nitems), index(0), nof_items(0) {
   assert(nitems > 0);
} // History<Item>::History

template<typename Item>
int History<Item>::max_size() const {
   return items.size();
} // History<Item>::max_size

template<typename Item>
int History<Item>::size() const {
   return nof_items;
} // History<Item>::size

template<typename Item>
const Item& History<Item>::operator[](int i) const {
   assert(i >= 0 && i < nof_items);
   // we are adding items.size to the left op of % to avoid
   // negative operands (effect not defined by ISO C++)
   return items[(items.size() + index - i - 1) % items.size()];
}; // History<Item>::operator[]

template<typename Item>
void History<Item>::add(const Item& item) {
   items[index] = item;
   index = (index + 1) % items.size();
   if (nof_items < items.size()) {
      nof_items += 1;
   }
} // History<Item>::add
